;; -*- coding: utf-8  -*-
;;; pdic-pkg.el --- Lookup supplement package for pdic-fuw
;; Copyright (C) 2019  Kazuhiro Ito <kzhr@d1.dion.ne.jp>

;; Author: Kazuhiro Ito <kzhr@d1.dion.nejp>
;; Target: ndebs, ndeb
;; Format: 1.1
;; Version: 1.0

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation,
;; Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Code:

(require 'lookup-package)

(defconst pdic-pkg-dictionary-options
  (list
   (cons ':adjusts
	 '(lookup-adjust-check-references
	   lookup-adjust-goto-min))
   (cons ':headings
	 '(ndeb-arrange-ignore
	   ndeb-arrange-decode-entity))
   (cons ':arranges
	 '(
	   ndeb-arrange-indent
	   pdic-pkg-arrange-face
	   pdic-pkg-arrange-font
	   ndeb-arrange-ignore
	   ndeb-arrange-prev-next
	   lookup-arrange-references
	   ndeb-arrange-decode-entity
	   lookup-arrange-default-headings
	   ndeb-arrange-fill-lines))
   ))

(defcustom pdic-pkg-font-scale-factor 1.2
  "Scale factor for pdic dictionary."
  :type 'number)

(defun pdic-pkg-put-text-face-properties (start end plist)
  ;; parameter of :height is relative scale when positive value.
  ;; Negative value means negated absolute size.
  (let (point props prop value)
    (while (< start end)
      (setq props (copy-sequence (get-text-property start 'face))
	    point (next-single-property-change start 'face nil end))
      (let ((plist plist))
	(while plist
	  (setq prop (car plist)
		value (cadr plist)
		plist (cddr plist))
	  (setq value (if (eq prop :height)
			  (if (> value 0)
			      (* (or (plist-get props :height) 1.0) value)
			    (- value))
			value))
	  (if props
	      (plist-put props prop value)
	    (setq props (list prop value)))))
      (put-text-property start point 'face props)
      (setq start point))))

(defun pdic-pkg-put-text-face-property (start end prop value)
  (pdic-pkg-put-text-face-properties start end (list prop value)))

(defun pdic-pkg-arrange-face (entry)
  (goto-char (point-min))
  (while (re-search-forward
	  "&lt;\\(b\\|i\\|s\\|u\\|small\\|big\\)&gt;\\(.*?\\)&lt;/\\1&gt;"
	  nil t)
    (let ((tag (match-string 1))
	  (next (match-beginning 0)))
      (cond
       ((string= tag "b")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2) :weight 'bold))
       ((string= tag "i")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2) :slant 'italic))
       ((string= tag "s")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2) :strike-through t))
       ((string= tag "u")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2) :underline t))
       ((string= tag "small")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2) :height
	 (/ pdic-pkg-font-scale-factor)))
       ((string= tag "big")
	(pdic-pkg-put-text-face-property
	 (match-beginning 0) (match-end 2)
	 :height pdic-pkg-font-scale-factor))
       )
      (delete-region (match-end 2) (match-end 0))
      (delete-region (match-beginning 0) (match-beginning 2))
      (goto-char (match-beginning 0))
      )))

(defun pdic-pkg-arrange-font-parms-parse (string)
  (let ((index 0) result name value number)
    (while (string-match "\\([a-zA-Z]+\\)=\"\\([^\"]+\\)\"" string index)
      (setq index (match-end 0)
	    name (match-string 1 string)
	    value (match-string 2 string))
      (cond
       ((string= name "color")
	(setq result (cons :foreground (cons value result))))
       ((string= name "bgcolor")
	(setq result (cons :background (cons value result))))
       ((and (string= name "size")
	     (string-match"-?[0-9]+" value))
	(setq number (string-to-number (match-string 0 value)))
	(setq result (cons :height
			   (cons (if (memq (aref value 0) '(?+ ?-))
				     (expt pdic-pkg-font-scale-factor
					   number)
				   ;; absolute size
				   (- (expt pdic-pkg-font-scale-factor
					    (abs (- number 3)))))
				 result))))))
    result))

(defun pdic-pkg-arrange-font (entry)
  (goto-char (point-min))
  (let (depth
	params point
	start start-end end end-start
	tmp1 tmp2)
    (while (setq point (re-search-forward "&lt;font \\(.+?\\)&gt;" nil t))
      (setq start (match-beginning 0)
	    start-end (match-end 0)
	    params (match-string 1)
	    depth 1)
      (while (null end)
	(setq tmp1 (progn (goto-char point)
			  (re-search-forward "&lt;font .+?&gt;" nil t)))
	(setq tmp2 (progn (goto-char point)
			  (re-search-forward "&lt;/font&gt;" nil t)))
	(cond
	 ((null tmp2)
	  (setq end (point-max)
		end-start end))
	 ((or (null tmp1)
	      (> tmp1 tmp2))
	  (if (< depth 2)
	      (setq end (match-end 0)
		    end-start (match-beginning 0))
	    (setq depth (1- depth))
	    (goto-char (setq point tmp2))))
	 (t ;; (< tmp1 tmp2)
	  (setq depth (1+ depth))
	  (goto-char (setq point tmp1)))
	 ))
      (pdic-pkg-put-text-face-properties
       start-end end-start (pdic-pkg-arrange-font-parms-parse params))
      (delete-region end-start end)
      (delete-region start start-end)
      (setq end nil)
      (goto-char start))))

(setq lookup-package-dictionary-options-alist
      (list (cons "pdic"
		  pdic-pkg-dictionary-options)))

;; pdic-pkg.el ends
